package ca.carleton.ccsl.deadbolt;

import android.app.IntentService;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.storage.IMountService;
import android.os.RemoteException;
import android.os.ServiceManager;
import android.os.Message;
import android.os.Messenger;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Log;

public class LockService extends IntentService {
    
    
    private static final String TAG = "Deadbolt.LockService";

      public LockService() {
		super("LockService");
	  }
    
	/**
     * Mount Service Interface Helper
     * 
     * 
     */
    public static IMountService getMountService() {
        IBinder service = ServiceManager.getService("mount");
        if (service != null) {
            return IMountService.Stub.asInterface(service);
        } else {
            Log.e(TAG, "Can't get mount service");
        }
        return null;
    }
    
    @Override
    protected void onHandleIntent(Intent intent) {
		int rc = -9;
        int mode = intent.getIntExtra("mode", -1);
		int flag = intent.getIntExtra("flag", -9);
		
		if (mode == 0) {
			rc = enDeadbolt(flag);
		} else if (mode == 1) {
			String passwd = intent.getStringExtra("passwd");
			rc = deDeadbolt(passwd, flag);
		} else {
			//bad mode
			Log.e(TAG, "Malformed Intent");
		}
		sendMsg(intent, rc);		
    }
    
	private int enDeadbolt(int flag) {
		Log.d(TAG, "Attempting to send command to Lock");
		int rc = -8;
        final IMountService mountService = getMountService();
        try {
			Thread.sleep(50);	//FIXME
            rc = mountService.cryptLock(flag);
        } catch (Exception e) {
            Log.e(TAG, "Something wrong en-deadbolting", e);
        }
    	return rc;
    }
    
	private int deDeadbolt(String passwd, int flag) {
		Log.d(TAG, "Attempting to send command to unlock");
		int rc = -8;
		
        final IMountService mountService = getMountService();
        try {
            rc = mountService.cryptUnlock(passwd, flag);
        } catch (Exception e) {
            Log.e(TAG, "Something wrong de-deadbolting", e);
        }
    	return rc;
    }
    
    private void sendMsg(Intent intent, int rc) {
    	Bundle extras = intent.getExtras();
		Messenger messenger = (Messenger) extras.get("MESSENGER");
		Message msg = Message.obtain();
		msg.arg1 = rc;
		try {
			messenger.send(msg);
		} catch (RemoteException e) {
			Log.e(TAG, "Exception sending message", e);
        }
    }
    
}
